from datetime import datetime, timedelta
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.gis.geoip import GeoIP
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
import re

BAN_TYPES = (
	('SI', _('IP address')),
    ('S', _('SteamID')),
)

class Ban(models.Model):
	bid = models.AutoField(_('id'), primary_key=True)
	player_ip = models.IPAddressField(_('player IP'))
	player_id = models.CharField(_('player ID'), max_length=35)
	player_nick = models.CharField(_('player nick'), max_length=100)
	admin_ip = models.IPAddressField(_('admin IP'))
	admin_id = models.CharField(_('admin ID'), max_length=35)
	admin_nick = models.CharField(_('admin nick'), max_length=100)
	ban_type = models.CharField(_('ban type'), max_length=10, choices=BAN_TYPES, default='S')
	ban_reason = models.CharField(_('ban reason'), max_length=100)
	cs_ban_reason = models.CharField(_('ban reason'), max_length=100, blank=True, help_text=_('unsure, you can leave it empty'))
	ban_created = models.IntegerField(_('ban created'), help_text=_('timestamp'))
	ban_length = models.IntegerField(_('ban length'), help_text=_('in minutes; -1 means unbanned but do NOT use this to unban player, use button on page or action in admin instead.'))
	server_ip = models.IPAddressField(_('server IP'))
	server_name = models.CharField(_('server name'), max_length=100)
	ban_kicks = models.IntegerField(_('kicks'), default=0)
	expired = models.IntegerField(_('expired'), default=0)
	imported = models.IntegerField(_('imported'), default=0)

	def __unicode__(self):
		return self.player_nick

	def get_creation_date(self):
		return datetime.fromtimestamp(self.ban_created)
	get_creation_date.short_description = _('date')

	def get_expiration_date(self):
		if not self.ban_length:
			return _('never')
		elif self.ban_length == -1:
			return _('unbanned')

		expiration = self.get_date() + timedelta(minutes=self.ban_length)
		today = timezone.now()

		if expiration < today:
			return _('expired')

		return expiration
	get_expiration_date.short_description = _('expires on')

	def get_length(self):
		if not self.ban_length:
			return _('permanent')
		elif self.ban_length == -1:
			return _('unbanned')
		else:
			return timedelta(minutes=self.ban_length)
	get_length.short_description = _('length')

	def is_active(self):
		if not self.ban_length:
			return True
		elif self.ban_length == -1 and self.expired == 1:
			return False
		return (datetime.fromtimestamp(self.ban_created) + timedelta(minutes=self.ban_length)) > datetime.today()
	is_active.short_description = _('active')
	is_active.boolean = True

	def get_country(self):
		return GeoIP().country(self.player_ip)

	def get_steam_community_id(self):
		if hasattr(self, '_community_id'):
			return self._community_id

		parts = re.match('^STEAM_(?P<x>\d+):(?P<y>[0-1]):(?P<z>\d+)$', self.player_id)

		if parts is None:
			return 0

		self._community_id = (int(parts.group('z')) * 2) + 76561197960265728 + int(parts.group('y'))
		return self._community_id

	class Meta:
		db_table = '%sbans' % settings.AMXBANS_DB_PREFIX
	
class Admin(models.Model):
	username = models.CharField(_('user name'), max_length=32)
	password = models.CharField(_('password'), max_length=50, blank=True)
	access = models.CharField(_('access'), max_length=32)
	flags = models.CharField(_('flags'), max_length=32)
	steamid = models.CharField(_('SteamID'), max_length=32, blank=True)
	nickname = models.CharField(_('nickname'), max_length=32)
	icq = models.IntegerField(_('ICQ'), blank=True, null=True)
	ashow = models.BooleanField(_('show in list'))
	created = models.IntegerField(_('created'), default=1)
	expired = models.IntegerField(_('expired'), default=0, blank=True)
	days = models.IntegerField(_('days'), default=0, blank=True)

	def __unicode__(self):
		return self.nickname

	def get_created(self):
		return datetime.fromtimestamp(self.created)

	def get_until(self):
		return _('infinite') if self.days == 0 else (self.get_created() + timedelta(days=self.days))

	class Meta:
		db_table = '%samxadmins' % settings.AMXBANS_DB_PREFIX

ADMIN_ROLES = (
	(0, _('owner')),
    (10, _('co-owner')),
	(20, _('manager')),
    (30, _('main admin')),
    (40, _('server admin')),
    (50, _('cheathunter')),
    (100, _('undefined')),
)

class AdminInfo(models.Model):
	admin = models.OneToOneField(Admin)
	user = models.ForeignKey(User, blank=True, null=True)
	role = models.SmallIntegerField(_('role'), choices=ADMIN_ROLES, default=100)
	show_role = models.BooleanField(_('show role behind nickname'), default=True)

	def __unicode__(self):
		return self.admin.nickname

class Server(models.Model):
	timestamp = models.IntegerField(_('last seen'))
	hostname = models.CharField(_('hostname'), max_length=100)
	address = models.CharField(_('address'), max_length=100)
	gametype = models.CharField(_('game mod'), max_length=32)
	rcon = models.CharField(_('rcon password'), max_length=32, blank=True)
	amxban_version = models.CharField(_('amxbans version'), max_length=32, blank=True)
	amxban_motd = models.CharField(_('amxbans motd'), max_length=250, blank=True)
	motd_delay = models.IntegerField(_('motd delay'), default=10)
	amxban_menu = models.IntegerField(_('amxbans menu'), default=1)
	reasons = models.IntegerField(_('reasons'))
	timezone_fixx = models.IntegerField(_('timezone fix'), default=0)

	admins = models.ManyToManyField(Admin, through='ServerAdmin')

	def __unicode__(self):
		return self.hostname

	def get_last_seen(self):
		return datetime.fromtimestamp(self.timestamp)
	get_last_seen.short_description = _('last seen')

	def get_admins(self):
		return self.admins.filter(access__contains='c')

	class Meta:
		db_table = '%sserverinfo' % settings.AMXBANS_DB_PREFIX

STATIC_BANTIME = (
	('yes', _('yes')),
	('no', _('no')),
)

class ServerAdmin(models.Model):
	admin = models.ForeignKey(Admin)
	server = models.ForeignKey(Server)
	custom_flags = models.CharField(_('custom access'), max_length=32, blank=True)
	use_static_bantime = models.CharField(_('use static ban time'), max_length=3, choices=STATIC_BANTIME, default='yes')

	def __unicode__(self):
		return "%s: %s" % (self.server, self.admin)

	class Meta:
		db_table = '%sadmins_servers' % settings.AMXBANS_DB_PREFIX
